/*
 *  Envelope.cpp
 *  audioOutputExample
 *
 *  Created by Zach Gage on 4/19/11.
 *  Copyright 2011 stfj. All rights reserved.
 *
 *
 * added make draw envelope method 
 * rux 2012
 */

#include "Envelope.h"

Envelope::Envelope(){
	size = 32;
	createDefaultEnvelope();

}

Envelope::~Envelope(){
	delete[] table; //delete the table when the envelope is deleted
}

void Envelope::setup(int tableSize, ofRectangle _rect){
    rect = _rect;
    
	if(tableSize != size){
		delete[] table;
		size = tableSize;
		createDefaultEnvelope();
	}
}

int Envelope::getTableSize(){
	return size;
}

void Envelope::setTableValue(int index, float value){
	if(index < size && index >= 0)
		table[index] = value;
	else
		cout<<"illegal index specified in setTableValue. You specified "<<index<<". Maximum size is "<<size<<endl;
}



void Envelope::makeDrawEnvelope(int x, int y){
    
    if (rect.inside(x,y)) {
        setTableValue(int(((float)x- rect.x)/ rect.width * getTableSize()), 1.0-(float)(y - rect.y)/ rect.height);
    }
}




float Envelope::getEnvelopeValue(float pct){
	
	if(pct < 0 || pct > 1) //if we try to get a percentage that is outside of 0->1 return 0
		return 0;
	
	
	int lowIndex = floor(pct*(size-1)); // the lower table index
	int highIndex = ceil(pct*(size-1)); //the upper table index (in case the pct falls between two indicies
	float pctBetween = pct*(size-1) - lowIndex; //how far it is between the indicies
	
	return table[lowIndex]*(1.0-pctBetween) + table[highIndex]*pctBetween; //return the percentage we are towards the top index * top index, + the pct we are away from the bottom index * the bottom index
}

void Envelope::draw(){
	
    ofFill();
    ofSetColor(160, 160, 160);
    ofRect(rect);
    
    ofNoFill();
    ofSetColor(0, 0, 0);
	ofBeginShape();
	for(int i=0;i< rect.width;i++){
		ofVertex(rect.x+i, (rect.y+rect.height)-getEnvelopeValue((float)i/rect.width)*rect.height);
	}
	ofEndShape(false);
}

void Envelope::createDefaultEnvelope(){
	table = new float[size];
	
	float pctMultiplier = PI/(float)size;
	
	for(int i=0;i<size;i++){
		table[i] = sin(i*pctMultiplier);
	}
}