#include "testApp.h"

//--------------------------------------------------------------
void testApp::setup(){

	ofBackground(255,255,255);

	// 2 output channels,
	// 0 input channels
	// 44100 samples per second
	// 256 samples per buffer
	// 4 num buffers (latency)

	ofSoundStreamSetup(2,0,this, 44100,256, 4);

	ofSetFrameRate(60);
	
	frequency = 120;
	position = 0;
	positionAdder = (frequency / 44100.0) * TWO_PI;
	
	envelopeLength = 2; //run the envelope over a two second span
	envelopePct = 0;
	envelopePctAdder = 1.0/(44100.0*envelopeLength); // how many samples per second * number of seconds
	
    
    // customize envelope
    myEnvelope.setup(32,ofRectangle(10,110,400,100));
    
    for(int i = 0; i < myEnvelope.getTableSize(); i++){
    
        myEnvelope.setTableValue(i, 0.0);

    }
    myEnvelope.setTableValue(0, 1.0);
    myEnvelope.setTableValue(1, 1.0);
    
    myEnvelope.setTableValue(myEnvelope.getTableSize()-1, 1.0);
	
    panel.setup("control", 570, 0, 300, 400);
	panel.addPanel("sound settings", 1, false);
	panel.setWhichPanel("sound settings");
	
	panel.addSlider("Envelope Size", "VAL_SIZE", 32,2,400, true);
	panel.addSlider("Envelope Length", "VAL_LENGTH", 2,0.01,2, false);
    panel.addToggle("Envelope Loop", "ENV_LOOP", false);
    panel.addSlider("Osci Frequency", "FREQ", 120, 50, 640, true);
	panel.loadSettings("settings.xml");
}


//--------------------------------------------------------------
void testApp::update(){
	panel.update();
	
	if(envelopeLength !=  panel.getValueF("VAL_LENGTH") ){
		envelopeLength = panel.getValueF("VAL_LENGTH");
		envelopePct = 0;
		envelopePctAdder = 1.0/(44100.0*envelopeLength); // how many samples per second * number of seconds
	}
    
    if (frequency != panel.getValueI("FREQ")) {
        frequency = panel.getValueI("FREQ");
        position = 0;
        positionAdder = (frequency/44100.0)*TWO_PI;
    }
	
	myEnvelope.setup(panel.getValueF("VAL_SIZE"), ofRectangle(10,110,400,100));
}

//--------------------------------------------------------------
void testApp::draw(){
    //ofSetColor(255, 255, 255);
	panel.draw();
	
	ofNoFill();
	ofSetLineWidth(2);
	myEnvelope.draw();

	ofSetColor(255,0,0);
	ofSetLineWidth(1);
	ofLine(myEnvelope.rect.x + envelopePct*myEnvelope.rect.width, myEnvelope.rect.y, myEnvelope.rect.x + envelopePct*myEnvelope.rect.width, myEnvelope.rect.y+ myEnvelope.rect.height); // draw a line to show where we are going through the envelope

}


//--------------------------------------------------------------
void testApp::keyPressed  (int key){

	
}

//--------------------------------------------------------------
void testApp::keyReleased  (int key){
    switch (key) {
        case 'q':
            envelopePctAdder = 1.0/(44100.0*envelopeLength); // how many samples per second * number of seconds
            envelopePct = 0;
            frequency = 100;
            positionAdder = (frequency / 44100.0) * TWO_PI;
            
            break;
        case 'r':
            for (int i = 0; i<myEnvelope.getTableSize(); i++) {
                myEnvelope.setTableValue(i, 0.0);
            }
        default:
            break;
    }
}

//--------------------------------------------------------------
void testApp::mouseMoved(int x, int y ){
	
	
}

//--------------------------------------------------------------
void testApp::mouseDragged(int x, int y, int button){
	panel.mouseDragged(x,y,button);
	
    myEnvelope.makeDrawEnvelope(x, y);

}

//--------------------------------------------------------------
void testApp::mousePressed(int x, int y, int button){
	panel.mousePressed(x,y,button);
	
    myEnvelope.makeDrawEnvelope(x, y);

}


//--------------------------------------------------------------
void testApp::mouseReleased(int x, int y, int button){
	panel.mouseReleased();
}

//--------------------------------------------------------------
void testApp::windowResized(int w, int h){

}
//--------------------------------------------------------------
void testApp::audioRequested 	(float * output, int bufferSize, int nChannels){

	for (int i = 0; i < bufferSize; i++){
		
		float sample = sin( position );
		position += positionAdder;
		
		//sample*=myEnvelope.getEnvelopeValue(envelopePct);
		
		output[i * 2 + 0] = sample*myEnvelope.getEnvelopeValue(envelopePct);			// left channel
		output[i * 2 + 1] = sample*myEnvelope.getEnvelopeValue(envelopePct);			// right channel
		
		envelopePct += envelopePctAdder;
				
        if (envelopePct > 1.0) {
            if(panel.getValueB("ENV_LOOP")){ // if we are past the end of the envelope, loop back to the start
                envelopePct -= 1.0;
                envelopePctAdder = 1.0/(44100.0*envelopeLength); // how many samples per second * number of seconds
            } else {
                envelopePctAdder = 0; // if we just want to run the envelope once we set the adder to 0
            }
        }
		
		
	}
	
	// because sin(..) of very large values is less precise, let's keep the position variable in the range of 0 to TWO_PI.  it's cyclical
	// so doing something like this, is very similar to %= for integer numbers. 
	while (position > TWO_PI) {
		position -= TWO_PI;	
	}
	
}

